
proc Set_V1290_Options {} {
    global Options V1290_Base

    load_Options

   if {![info exists Options(v1290.channels)]} {set Options(v1290.channels) 4096}
   if {![info exists Options(v1290.offset)]} {set Options(v1290.offset) 0}
   if {![info exists Options(v1290.shift)]} {set Options(v1290.shift) 4}
   if {![info exists Options(v1290_GroupBase)]} {set Options(v1290_GroupBase) $V1290_Base}
   set V1290_Base   $Options(v1290_GroupBase)

   if {![info exists Options(v1290.acqmode)]} {set Options(v1290.acqmode) 0}
   if {![info exists Options(v1290.windowwidth)]} {set Options(v1290.windowwidth) 160}
   if {![info exists Options(v1290.windowoffset)]} {set Options(v1290.windowoffset) -150}

    update_Options
}


proc Dump_Raw_VME_Data_V1290 {} {

   set info ""

   set maxmodule [inform vmecontrol inquire v1290count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\nV1290
#$i\n"
      execute-command 19 "Getting Raw Data for V1290#$i" "1 $i" 2
      set data [read-register EXEC.Response]
      
     foreach word $data {
        set word 0x$word
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 30 & 3]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 19 & 0x7f]
             set ed [expr $word >> 26 & 1]
             set tdc [expr $word  & 0x3ffff]
             append info "[format "0x%8x    %8s   channel= %5i; edge= %3i; time= 0x%5x" $word $event_type $channel $ed $tdc]"
            }
            1 {
             set event_type "header"
             set geo [expr $word & 0x1f]
             set counter [expr $word >> 5 & 0x3fffff]
             append info "[format "0x%8x    %8s   module= %5i; event number= %8i" $word $event_type $geo $counter]"
            }
            2 {
             set event_type "trailer"
             set geo [expr $word & 0x1f]
             set status [expr $word >> 24 & 7]
             set counter [expr $word >> 5 & 0xffff]
             append info "[format "0x%8x    %8s   module= %5i; status=%2i; event data counter= %8i" $word $event_type $geo $status $counter]"
            }
            3 {
             append info "$word - assumed end of data\n"
             clear-busy
             set EGreport $EGreport_Save
             return $info
            }
         }
         append info "\n"
     }
   }
   return $info
}

proc define_V1290_config {} {
   global VMEserver
   global V1290_Base

   set maxmodule [inform vmecontrol inquire v1290count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "1290 0 [format "0x%x" $V1290_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v1290#$i.Output_Buffer'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v767#$i.Output_Buffer"
          clear-footer
          error $m
      }
      DefineConfig "1290 $i [string range [first $m] 2 end]"
   }
}

proc create_V1290_spectra {} {
    global SpectrumServer V1290_Base
    global Options

   set maxmodule [inform vmecontrol inquire v1290count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 128

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module v1290#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v1290#$i.tdc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(v1290.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum v1290#$i.tdc$j"
          clear-footer
          error $m
      } else {
         if {$j < 64} then {
             set group [expr $V1290_Base + (($i - 1) * 2)]
             set item $j
         } else {
             set group [expr $V1290_Base + (($i - 1) * 2) + 1]
             set item [expr $j - 64]
         }
          map_spectrum  "$spectrumname $group $item $Options(v1290.offset) $Options(v1290.shift)"
      }
    }
   }
}

proc do_V1290_setup {} {
    global Options

   set maxmodule [inform vmecontrol inquire v1290count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   if {$maxmodule == 0} {return}
   
   starttask v1290

   for {set i 1} {$i <= $maxmodule} {incr i} {

       inform v1290 click-select-module $i
               
#    acquisition mode:
#      set trigger matching
       inform v1290 do_AcqMode_opcode OPT00

#    trigger options:
#      set window width 
#      set window offset 
       inform v1290 do_Trig_opcode OPT10 $Options(v1290.windowwidth)
       inform v1290 do_Trig_opcode OPT11 $Options(v1290.windowoffset)

#      set extra search margin 0
#       set reject margin 0
       inform v1290 do_Trig_opcode OPT12 0
       inform v1290 do_Trig_opcode OPT13 0

#     enable subtraction of trigger time
       inform v1290 do_Trig_opcode OPT14 0

#    edge detection
#      set edge detection  only leading
#      set LSB 800ps
       inform v1290 do_Edge_opcode OPT22 1
       inform v1290 do_Edge_opcode OPT24 0

#    readout 
#    disable TDC header & EOB
#    max hits 128
       inform v1290 do_Readout_opcode OPT31 0
       inform v1290 do_Readout_opcode OPT33 8
   }
}
